#ifndef GCODER_H
#define GCODER_H

#include <deque>
#include <utility>
#include "vec.h"
#include <QByteArray>
#include <QMutex>

class Mesh;

class GCoder
{
private:
    GCoder();

public:

    void draw();

    void clear();

    void computeLayers(Mesh *mesh);

	QByteArray computeGCodeFromLayers() const;

	std::pair<Vec3, Vec3> parseGCode(const QByteArray &gcode);
	double estimateTimeToPrint(const QByteArray &gcode) const;

private:
	void lock() const;
	void unlock() const;

private:
    std::deque<std::pair<double, std::vector<Vec2d> > > layers;
    std::vector<Vec3f> cached_lines;
    std::vector<Vec3f> cached_points;
	std::vector<Vec3f> cached_color;
	mutable QMutex mutex;

private:
    static GCoder *pInstance;
public:
    static GCoder *instance();
};

#endif // GCODER_H
